# *******************************************************
# ****** Import libraries ******
# *******************************************************

from ..utils.Preprocessing_data import Train_reducer, Train_inverter, Reduce, Inverse

# *******************************************************
# ****** Up_search_space_params ******
# *******************************************************

def Up_search_space_params(x, x_tau, d_nc, dims, problem_type, n_components, reducer, inverter, inverter_transform):

    if reducer == "no":
        x_tau_ = x_tau
        reducer_trained = None
        inverter_ = None
    else:
        x_tau_, reducer_trained = Train_reducer(x, d_nc, dims, problem_type, n_components, reducer) 
        if inverter_transform == "no":
            inverter_ = Train_inverter(x, x_tau, dims, inverter)
        elif inverter_transform == "yes":
            inverter_ = reducer_trained
        else:
            pass
        
    return x_tau_, reducer_trained, inverter_


# *******************************************************
# Import libraries
# *******************************************************

import numpy as np
from scipy.optimize import minimize

# *******************************************************
# ****** Querry ******
# *******************************************************

def Querry(x_l, x_u, bnds, dims, af_params, constraints_method, sense, model, models_const, engine, Acq_fun):

    # Acquisition function for the optimization
    def Acquisition_function_opt(x, af_params, sense, model, models_const, engine):

        if sense == "maximize":
            af_opt = -Acq_fun(x.reshape(1,-1), af_params, constraints_method, model, models_const, engine)
        elif sense == "minimize":
            af_opt = Acq_fun(x.reshape(1,-1), af_params, constraints_method, model, models_const, engine)
        
        return af_opt
    
    # Main program
    
    res = minimize(Acquisition_function_opt, x0=np.random.uniform(x_l, x_u, (1,dims)).flatten(), args=(af_params, sense, model, models_const, engine), method='BFGS', bounds=bnds)
    x_opt = res.x

    return x_opt